/*
    Bantumi
    Copyright 2005, 2006 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsj
    martin@martin.st
*/

#include <avkon.hrh>
#include <aknnotewrappers.h>
#include <stringloader.h>
#include <flogger.h>
#include <bantumigl.rsg>

#include "bantumi.pan"
#include "bantumiappui.h"
#include "bantumiappview.h"
#include "bantumi.hrh"
#include "bantumi.h"
#include "symbianbt.h"
#include "connwrap.h"
#include "symbiantcp.h"
#include <eikmenub.h>

void showDialog(CSymbianConnection* aConn, TBool aClient);

void CBantumiAppUi::ConstructL() {
	BaseConstructL();
	iAppView = CBantumiAppView::NewL(ClientRect(), ETrue);
	AddToStackL(iAppView);
}

CBantumiAppUi::CBantumiAppUi() {
	iPortNumber = 8471;
}

CBantumiAppUi::~CBantumiAppUi() {
	if (iAppView) {
		RemoveFromStack(iAppView);
		delete iAppView;
		iAppView = NULL;
	}
}

void CBantumiAppUi::HandleCommandL(TInt aCommand) {
	CBantumiAppView* appView = static_cast<CBantumiAppView*>(iAppView);
	switch(aCommand) {
	case EEikCmdExit:
	case EAknSoftkeyExit:
	case EAknSoftkeyBack:
		DoExit();
		break;

	case EBantumiSoftkey1:
		if (appView->iBantumi)
			appView->iBantumi->pressed(START);
		break;

	case EBantumiSoftkey2:
		if (appView->iBantumi)
			appView->iBantumi->pressed(ESCAPE);
		break;

	case EBantumiHost:
		if (iConnectionType == EBantumiBluetooth)
			iConn->SetDelegate(CBTConnection::NewL());
		else
			iConn->SetDelegate(CTCPConnection::NewL());
		HandleForegroundEventL(EFalse);
		appView->EnableCallback(EFalse);
		showDialog(iConn->GetDelegate(), EFalse);
		appView->EnableCallback(ETrue);
		iConn = NULL;
		break;

	case EBantumiJoin:
		if (iConnectionType == EBantumiBluetooth)
			iConn->SetDelegate(CBTConnection::NewL());
		else
			iConn->SetDelegate(CTCPConnection::NewL());
		HandleForegroundEventL(EFalse);
		appView->EnableCallback(EFalse);
		showDialog(iConn->GetDelegate(), ETrue);
		appView->EnableCallback(ETrue);
		iConn = NULL;
		break;

	case EBantumiCancel:
		iConn->CancelConn();
		iConn = NULL;
		break;

	case EBantumiTCPIP:
		// don't show any host/join dialog in this case
		iConnectionType = aCommand;
		iConn->SetDelegate(CTCPConnection::NewL());
		HandleForegroundEventL(EFalse);
		appView->EnableCallback(EFalse);
		showDialog(iConn->GetDelegate(), ETrue);
		appView->EnableCallback(ETrue);
		iConn = NULL;
		break;

	case EBantumiBluetooth: {
		iConnectionType = aCommand;
		CEikMenuBar *menubar = CEikonEnv::Static()->AppUiFactory()->MenuBar();
//		menubar->StopDisplayingMenuBar();
		menubar->SetMenuTitleResourceId(R_BANTUMI_MULTIPLAYER_MENUBAR);
		menubar->TryDisplayMenuBarL();
		}
		break;

	default:
		Panic(EBantumiUi);
		break;
	}
}

void CBantumiAppUi::ProcessCommandL(TInt aCommand) {
	// when selecting something from the popupmenu,
	// the functions are called in the order ProcessCommandL, SetEmphasis, HandleCommandL
	switch(aCommand) {
	case EBantumiHost:
	case EBantumiJoin:
	case EBantumiBluetooth:
	case EBantumiTCPIP:
	case EBantumiCancel:
		iMenuSelected = ETrue;
		break;

	default:
		break;
	}

	CAknAppUi::ProcessCommandL(aCommand);

}

void CBantumiAppUi::SetEmphasis(CCoeControl* aMenuControl, TBool aEmphasis) {
	if (aEmphasis) iMenuSelected = EFalse;

	if (iConn && !aEmphasis && !iMenuSelected) {
		// closing menu, but no action was selected in the menu
		if (iConn) {
			iConn->CancelConn();
			iConn = NULL;
		}
	}
	CGLAppUi::SetEmphasis(aMenuControl, aEmphasis);
}

// workaround for the winscw compiler, which on s60 3.0 seems to
// leak memory (causes alloc panic on exit) if there exists a
// TRAP in the call stack to Exit()

void CBantumiAppUi::DoSave() {
	TRAPD(err, SaveL());
}

void CBantumiAppUi::DoExit() {
	DoSave();
	Exit();
}


void CBantumiAppUi::SetConnection(CConnectionWrapper* aConn) {
	iConn = aConn;
}

CBantumiAppView* CBantumiAppUi::AppView() const {
	return static_cast<CBantumiAppView*>(iAppView);
}

void CBantumiAppUi::SetHostname(const TDesC& aName) {
	iHostname = aName;
}

const TDesC& CBantumiAppUi::GetHostname() const {
	return iHostname;
}

void CBantumiAppUi::SetPortNumber(TInt aPort) {
	iPortNumber = aPort;
}

TInt CBantumiAppUi::GetPortNumber() const {
	return iPortNumber;
}


